@@ -1,18 +1,17 @@
 /*
  * eab-contact-formatter.c
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) version 3.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with the program; if not, see <http://www.gnu.org/licenses/>
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -699,6 +698,29 @@ render_personal_column (EABContactFormatter *formatter,
 	g_string_free (accum, TRUE);
 }
 
+static void
+render_other_column (EABContactFormatter *formatter,
+                     EContact *contact,
+                     GString *buffer)
+{
+	GString *accum = g_string_new ("");
+
+	accum_address (accum, contact, _("Address"), E_CONTACT_ADDRESS_OTHER, E_CONTACT_ADDRESS_LABEL_OTHER);
+	if (formatter->priv->render_maps)
+		accum_address_map (accum, contact, E_CONTACT_ADDRESS_OTHER);
+
+	if (accum->len > 0) {
+		g_string_append_printf (
+			buffer,
+			"<div class=\"column\" id=\"contact-other\">"
+			"<h3>%s</h3>"
+			"<table border=\"0\" cellspacing=\"5\">%s</table>"
+			"</div>", _("Other"), accum->str);
+	}
+
+	g_string_free (accum, TRUE);
+}
+
 static void
 render_footer (EABContactFormatter *formatter,
                EContact *contact,
@@ -736,6 +758,7 @@ render_contact (EABContactFormatter *formatter,
 	render_contact_column (formatter, contact, buffer);
 	render_work_column (formatter, contact, buffer);
 	render_personal_column (formatter, contact, buffer);
+	render_other_column (formatter, contact, buffer);
 	g_string_append (buffer, "</div>");
 
 	render_footer (formatter, contact, buffer);
