@@ -1,18 +1,17 @@
 /*
  * Evolution calendar - Framework for a calendar component editor dialog
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) version 3.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with the program; if not, see <http://www.gnu.org/licenses/>
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  *
  * Authors:
@@ -159,6 +158,9 @@ static const gchar *ui =
 "      <menuitem action='close'/>"
 "    </menu>"
 "    <menu action='edit-menu'>"
+"      <menuitem action='undo'/>"
+"      <menuitem action='redo'/>"
+"      <separator/>"
 "      <menuitem action='cut-clipboard'/>"
 "      <menuitem action='copy-clipboard'/>"
 "      <menuitem action='paste-clipboard'/>"
@@ -180,6 +182,9 @@ static const gchar *ui =
 "    <toolitem action='save'/>\n"
 "    <toolitem action='print'/>\n"
 "    <separator/>"
+"    <toolitem action='undo'/>"
+"    <toolitem action='redo'/>"
+"    <separator/>"
 "    <placeholder name='content'/>\n"
 "  </toolbar>"
 "</ui>";
@@ -1289,6 +1294,21 @@ static GtkActionEntry core_entries[] = {
 	  N_("Select all text"),
 	  NULL },  /* Handled by EFocusTracker */
 
+	{ "undo",
+	  GTK_STOCK_UNDO,
+	  NULL,
+	  "<Control>z",
+	  N_("Undo"),
+	  NULL },  /* Handled by EFocusTracker */
+
+	{ "redo",
+	  GTK_STOCK_REDO,
+	  NULL,
+	  "<Control>y",
+	  N_("Redo"),
+	  NULL },  /* Handled by EFocusTracker */
+
+
 	/* Menus */
 
 	{ "classification-menu",
@@ -2200,6 +2220,12 @@ comp_editor_init (CompEditor *editor)
 	action = comp_editor_get_action (editor, "select-all");
 	e_focus_tracker_set_select_all_action (focus_tracker, action);
 
+	action = comp_editor_get_action (editor, "undo");
+	e_focus_tracker_set_undo_action (focus_tracker, action);
+
+	action = comp_editor_get_action (editor, "redo");
+	e_focus_tracker_set_redo_action (focus_tracker, action);
+
 	priv->focus_tracker = focus_tracker;
 
 	/* Fine Tuning */
