@@ -1,18 +1,17 @@
 /*
  * Evolution calendar - Main page of the task editor dialog
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) version 3.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with the program; if not, see <http://www.gnu.org/licenses/>
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  *
  * Authors:
@@ -922,6 +921,11 @@ task_page_fill_widgets (CompEditorPage *page,
 
 	sensitize_widgets (tpage);
 
+	e_widget_undo_reset (priv->summary);
+	e_widget_undo_reset (priv->categories);
+	e_widget_undo_reset (priv->web_page_entry);
+	e_widget_undo_reset (priv->description);
+
 	return TRUE;
 }
 
@@ -2653,6 +2657,7 @@ task_page_construct (TaskPage *tpage,
 	EShell *shell;
 	CompEditor *editor;
 	ESourceRegistry *registry;
+	EFocusTracker *focus_tracker;
 	TaskPagePrivate *priv;
 	GtkComboBox *combo_box;
 	GtkListStore *list_store;
@@ -2662,6 +2667,7 @@ task_page_construct (TaskPage *tpage,
 
 	editor = comp_editor_page_get_editor (COMP_EDITOR_PAGE (tpage));
 	shell = comp_editor_get_shell (editor);
+	focus_tracker = comp_editor_get_focus_tracker (editor);
 
 	priv = tpage->priv;
 	priv->meeting_store = g_object_ref (meeting_store);
@@ -2685,6 +2691,10 @@ task_page_construct (TaskPage *tpage,
 	}
 
 	e_spell_text_view_attach (GTK_TEXT_VIEW (priv->description));
+	e_widget_undo_attach (priv->summary, focus_tracker);
+	e_widget_undo_attach (priv->categories, focus_tracker);
+	e_widget_undo_attach (priv->web_page_entry, focus_tracker);
+	e_widget_undo_attach (priv->description, focus_tracker);
 
 	combo_box = GTK_COMBO_BOX (priv->organizer);
 	model = gtk_combo_box_get_model (combo_box);
