@@ -1,18 +1,17 @@
 /*
  * e-charset-combo-box.c
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) version 3.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with the program; if not, see <http://www.gnu.org/licenses/>
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  *
  * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
@@ -220,32 +219,6 @@ charset_combo_box_get_property (GObject *object,
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 }
 
-static void
-charset_combo_box_constructed (GObject *object)
-{
-	ECharsetComboBoxPrivate *priv;
-	GtkRadioAction *radio_action;
-	GSList *group;
-
-	priv = E_CHARSET_COMBO_BOX_GET_PRIVATE (object);
-
-	/* Chain up to parent's constructed() method. */
-	G_OBJECT_CLASS (e_charset_combo_box_parent_class)->constructed (object);
-
-	radio_action = priv->other_action;
-	group = gtk_radio_action_get_group (radio_action);
-
-	e_action_combo_box_set_action (
-		E_ACTION_COMBO_BOX (object), radio_action);
-
-	e_action_combo_box_add_separator_after (
-		E_ACTION_COMBO_BOX (object), g_slist_length (group));
-
-	g_signal_connect (
-		object, "notify::charset",
-		G_CALLBACK (charset_combo_box_notify_charset_cb), NULL);
-}
-
 static void
 charset_combo_box_dispose (GObject *object)
 {
@@ -309,7 +282,6 @@ e_charset_combo_box_class_init (ECharsetComboBoxClass *class)
 	object_class = G_OBJECT_CLASS (class);
 	object_class->set_property = charset_combo_box_set_property;
 	object_class->get_property = charset_combo_box_get_property;
-	object_class->constructed = charset_combo_box_constructed;
 	object_class->dispose = charset_combo_box_dispose;
 	object_class->finalize = charset_combo_box_finalize;
 
@@ -371,6 +343,17 @@ e_charset_combo_box_init (ECharsetComboBox *combo_box)
 	g_object_set_data (G_OBJECT (radio_action), "charset", (gpointer) "");
 
 	gtk_radio_action_set_group (radio_action, group);
+	group = gtk_radio_action_get_group (radio_action);
+
+	e_action_combo_box_set_action (
+		E_ACTION_COMBO_BOX (combo_box), radio_action);
+
+	e_action_combo_box_add_separator_after (
+		E_ACTION_COMBO_BOX (combo_box), g_slist_length (group));
+
+	g_signal_connect (
+		combo_box, "notify::charset",
+		G_CALLBACK (charset_combo_box_notify_charset_cb), NULL);
 
 	combo_box->priv->other_action = radio_action;
 }
